/* -*- tab-width: 4 -*-
 *
 * Electric(tm) VLSI Design System
 *
 * File: ErrorRepositoryLogger.java
 *
 * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
 *
 * Electric(tm) is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Electric(tm) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Electric(tm); see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, Mass 02111-1307, USA.
 */
package com.sun.electric.api.minarea.errorLogging;

import java.awt.Shape;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import com.sun.electric.api.minarea.ErrorLogger;
import com.sun.electric.api.minarea.geometry.Shapes;
import java.awt.geom.Point2D;

/**
 * @author Felix Schmidt
 * 
 */
public class ErrorRepositoryLogger implements ErrorLogger {

    private class MinAreaViolation {

        private final long minArea;
        private final int x;
        private final int y;
        private final Shape shape;

        public MinAreaViolation(long minArea, int x, int y, Shape shape) {
            this.minArea = minArea;
            this.x = x;
            this.y = y;
            this.shape = shape;
            if (shape != null) {
                Point2D maxVertex = Shapes.maxVertex(shape);
                if (x != maxVertex.getX() || y != maxVertex.getY()) {
                    throw new IllegalArgumentException();
                }
            }
        }

        @Override
        public String toString() {
            return new StringBuilder().append("(").append(x).append(",").append(y).append(")").toString();
        }
    }
    private List<ErrorRepositoryLogger.MinAreaViolation> violations = new LinkedList<ErrorRepositoryLogger.MinAreaViolation>();

    /*
     * (non-Javadoc)
     * 
     * @see com.sun.electric.api.minarea.MinAreaChecker.ErrorLogger#
     * reportMinAreaViolation(long, int, int)
     */
    public synchronized void reportMinAreaViolation(long minArea, int x, int y, Shape shape) {
        violations.add(new MinAreaViolation(minArea, x, y, shape));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sun.electric.api.minarea.MinAreaChecker.ErrorLogger#printReports
     * ()
     */
    public void printReports() {
        System.out.println("***********************************************");
        if (violations.isEmpty()) {
            System.out.println("No DRC violation found: Good Job!");
        } else {
            System.out.println("DRC Min-Area Violations: " + violations.size());
            System.out.println();
            sortViolations();
            for (MinAreaViolation violation : violations) {
                System.out.println("reportMinAreaViolation("
                        + violation.toString() + ", " + violation.minArea
                        + ");");
            }
            System.out.println("***********************************************");
        }
    }

    private synchronized void sortViolations() {
        Collections.sort(violations, new Comparator<MinAreaViolation>() {

            public int compare(MinAreaViolation v1, MinAreaViolation v2) {
                if (v1.x > v2.x) {
                    return 1;
                }
                if (v1.x < v2.x) {
                    return -1;
                }
                if (v1.y > v2.y) {
                    return 1;
                }
                if (v1.y < v2.y) {
                    return -1;
                }
                return 0;
            }
        });
    }
}
